/// <reference types="node" />
import type { SecretsManager, Event, StoredEvent, EventThreadData, StoredEventPointer, StoredEventBatchPointer, Eventstore as CoreEventstore, InputCursor, Cursor, SecretRecord, OldSecretRecord, OldEvent, ReplicationState, Monitoring } from '@resolve-js/core';
import stream from 'stream';
import { MAINTENANCE_MODE_AUTO, MAINTENANCE_MODE_MANUAL } from './constants';
import * as t from 'io-ts';
import * as iotsTypes from 'io-ts-types';
export type { StoredEvent, EventThreadData, StoredEventPointer, StoredEventBatchPointer, InputCursor, Cursor, };
export declare function validate<T extends t.Type<any>>(schema: T, params: any, errorName?: string): t.TypeOf<T>;
declare type DefinedType<T extends any> = undefined extends Extract<T, undefined> ? Exclude<T, undefined> : T;
export declare type UnbrandProps<T extends any> = {
    [Property in keyof T]: DefinedType<T[Property]> extends string & t.Brand<infer B> ? string | Extract<T[Property], undefined> : DefinedType<T[Property]> extends t.Branded<infer S, infer B> ? S | Extract<T[Property], undefined> : T[Property];
};
export declare type InputEvent = Event;
export declare type VersionlessEvent = Omit<InputEvent, 'aggregateVersion'>;
export type { SecretRecord, OldSecretRecord, OldEvent };
export type { ReplicationState };
export declare function getInitialReplicationState(): ReplicationState;
export declare type CheckForResourceError = (errors: Error[]) => void;
declare type DeleteSecret = SecretsManager['deleteSecret'];
declare type GetSecret = SecretsManager['getSecret'];
declare type SetSecret = SecretsManager['setSecret'];
declare type ShapeEvent = (event: any, additionalFields?: any) => StoredEvent;
export declare type ValidateEventFilter = (filter: EventFilter) => void;
declare const TimestampFilterSchema: t.IntersectionC<[t.IntersectionC<[t.TypeC<{
    limit: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    eventTypes: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    aggregateIds: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    eventsSizeLimit: t.BrandC<t.NumberC, t.IntBrand>;
}>]>, t.PartialC<{
    startTime: t.BrandC<t.NumberC, t.IntBrand>;
    finishTime: t.BrandC<t.NumberC, t.IntBrand>;
}>]>;
declare type TimestampFilterChecked = t.TypeOf<typeof TimestampFilterSchema>;
export declare type TimestampFilter = UnbrandProps<TimestampFilterChecked>;
declare const CursorFilterSchema: t.IntersectionC<[t.IntersectionC<[t.TypeC<{
    limit: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    eventTypes: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    aggregateIds: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    eventsSizeLimit: t.BrandC<t.NumberC, t.IntBrand>;
}>]>, t.TypeC<{
    cursor: t.UnionC<[t.StringC, t.NullC]>;
}>]>;
declare type CursorFilterChecked = t.TypeOf<typeof CursorFilterSchema>;
export declare type CursorFilter = UnbrandProps<CursorFilterChecked>;
declare const EventFilterSchemaSimple: t.IntersectionC<[t.IntersectionC<[t.TypeC<{
    limit: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    eventTypes: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    aggregateIds: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    eventsSizeLimit: t.BrandC<t.NumberC, t.IntBrand>;
}>]>, t.PartialC<{
    startTime: t.BrandC<t.NumberC, t.IntBrand>;
    finishTime: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    cursor: t.UnionC<[t.StringC, t.NullC]>;
}>]>;
export declare const EventFilterSchema: t.Type<UnbrandProps<{
    limit: t.Branded<number, t.IntBrand>;
} & {
    eventTypes?: string[] | null | undefined;
    aggregateIds?: string[] | null | undefined;
    eventsSizeLimit?: t.Branded<number, t.IntBrand> | undefined;
} & {
    startTime?: t.Branded<number, t.IntBrand> | undefined;
    finishTime?: t.Branded<number, t.IntBrand> | undefined;
} & {
    cursor?: string | null | undefined;
}>, UnbrandProps<{
    limit: t.Branded<number, t.IntBrand>;
} & {
    eventTypes?: string[] | null | undefined;
    aggregateIds?: string[] | null | undefined;
    eventsSizeLimit?: t.Branded<number, t.IntBrand> | undefined;
} & {
    startTime?: t.Branded<number, t.IntBrand> | undefined;
    finishTime?: t.Branded<number, t.IntBrand> | undefined;
} & {
    cursor?: string | null | undefined;
}>, unknown>;
declare type EventFilterChecked = t.TypeOf<typeof EventFilterSchemaSimple>;
export declare type EventFilter = UnbrandProps<EventFilterChecked>;
export declare type LatestEventFilter = Pick<EventFilter, 'aggregateIds' | 'eventTypes'>;
export declare type EventLoaderFilter = Omit<CursorFilter, 'limit' | 'eventsSizeLimit'>;
export declare type SecretFilter = {
    idx?: SecretRecord['idx'] | null;
    skip?: number;
    limit: number;
    ids?: Array<SecretRecord['id']> | null;
    includeDeleted?: boolean;
};
export declare type SecretsWithIdx = {
    idx: SecretRecord['idx'] | null;
    secrets: SecretRecord[];
};
export declare type GatheredSecrets = {
    existingSecrets: SecretRecord[];
    deletedSecrets: Array<SecretRecord['id']>;
};
export declare function isTimestampFilter(filter: EventFilter): filter is TimestampFilter;
export declare function isCursorFilter(filter: EventFilter): filter is CursorFilter;
export declare type RemoveFirstType<T extends any[]> = T extends [infer _, ...infer R] ? R : never;
export declare type PromiseResultType<T extends any> = T extends Promise<infer R> ? R : T;
export declare type PoolMethod<ConfiguredProps extends {}, M extends (...args: any) => any> = (pool: AdapterBoundPool<ConfiguredProps>, ...args: Parameters<M>) => ReturnType<M>;
export declare const AdapterConfigSchema: t.PartialC<{
    snapshotBucketSize: t.BrandC<t.NumberC, t.IntBrand>;
}>;
declare type AdapterConfigChecked = t.TypeOf<typeof AdapterConfigSchema>;
export declare type AdapterConfig = UnbrandProps<AdapterConfigChecked>;
export declare const AdapterTableNamesSchema: t.PartialC<{
    secretsTableName: iotsTypes.NonEmptyStringC;
    eventsTableName: iotsTypes.NonEmptyStringC;
    snapshotsTableName: iotsTypes.NonEmptyStringC;
    subscribersTableName: iotsTypes.NonEmptyStringC;
}>;
declare type AdapterTableNamesChecked = t.TypeOf<typeof AdapterTableNamesSchema>;
export declare type AdapterTableNames = UnbrandProps<AdapterTableNamesChecked>;
export declare type AdapterTableNamesProps = Required<AdapterTableNames>;
export declare type AdapterPoolPrimalProps = {
    disposed: boolean;
    validateEventFilter: ValidateEventFilter;
    maybeThrowResourceError: CheckForResourceError;
    bucketSize: number;
    counters: Map<string, number>;
    getNextCursor: CoreEventstore['getNextCursor'];
    monitoring: Monitoring | null;
};
export declare type AdapterPoolBoundProps = Adapter & AdapterPoolPrivateBoundProps;
export declare type AdapterPrimalPool<ConfiguredProps extends {}> = AdapterPoolPrimalProps & Partial<ConfiguredProps> & Partial<AdapterPoolBoundProps>;
export declare type AdapterConfiguredPool<ConfiguredProps extends {}> = AdapterPoolPrimalProps & ConfiguredProps & Partial<AdapterPoolBoundProps>;
export declare type AdapterBoundPool<ConfiguredProps extends {}> = AdapterPoolPrimalProps & ConfiguredProps & AdapterPoolBoundProps;
export declare type AdapterPoolPrivateBoundProps = {
    injectEvent: (event: StoredEvent) => Promise<void>;
    injectEvents: (events: StoredEvent[]) => Promise<void>;
    injectSecret: (secretRecord: SecretRecord) => Promise<void>;
    loadEventsByTimestamp: (filter: TimestampFilter) => Promise<StoredEventBatchPointer>;
    loadEventsByCursor: (filter: CursorFilter) => Promise<StoredEventBatchPointer>;
    deleteSecret: DeleteSecret;
    getSecret: GetSecret;
    setSecret: SetSecret;
    shapeEvent: ShapeEvent;
    initEvents: () => Promise<any[]>;
    initSecrets: () => Promise<any[]>;
    initFinal: () => Promise<any[]>;
    dropEvents: () => Promise<any[]>;
    dropSecrets: () => Promise<any[]>;
    dropFinal: () => Promise<any[]>;
    getEventLoaderNative?: (filter: EventLoaderFilter) => Promise<EventLoader>;
};
export declare type MAINTENANCE_MODE = typeof MAINTENANCE_MODE_AUTO | typeof MAINTENANCE_MODE_MANUAL;
export declare type ImportOptions = {
    byteOffset: number;
    maintenanceMode: MAINTENANCE_MODE;
};
export declare type ImportEventsStream = stream.Writable & {
    readonly byteOffset: number;
    readonly savedEventsCount: number;
};
export declare type ExportOptions = {
    cursor: InputCursor;
    maintenanceMode: MAINTENANCE_MODE;
    bufferSize: number;
    preferRegularEventLoader: boolean;
};
export declare type ExportEventsStream = stream.Readable & {
    readonly cursor: InputCursor;
    readonly isBufferOverflow: boolean;
    readonly isEnd: boolean;
    readonly preferRegularEventLoader: boolean;
};
export declare type ImportSecretsOptions = {
    maintenanceMode: MAINTENANCE_MODE;
};
export declare type ExportSecretsOptions = {
    idx: number | null;
    maintenanceMode: MAINTENANCE_MODE;
};
export interface CommonAdapterFunctions<ConfiguredProps extends {}> {
    maybeThrowResourceError: CheckForResourceError;
    validateEventFilter: ValidateEventFilter;
    loadEvents: PoolMethod<ConfiguredProps, Adapter['loadEvents']>;
    importEventsStream: PoolMethod<ConfiguredProps, Adapter['importEvents']>;
    exportEventsStream: PoolMethod<ConfiguredProps, Adapter['exportEvents']>;
    incrementalImport: PoolMethod<ConfiguredProps, Adapter['incrementalImport']>;
    getNextCursor: CoreEventstore['getNextCursor'];
    importSecretsStream: PoolMethod<ConfiguredProps, Adapter['importSecrets']>;
    exportSecretsStream: PoolMethod<ConfiguredProps, Adapter['exportSecrets']>;
    init: PoolMethod<ConfiguredProps, Adapter['init']>;
    drop: PoolMethod<ConfiguredProps, Adapter['drop']>;
    gatherSecretsFromEvents: PoolMethod<ConfiguredProps, Adapter['gatherSecretsFromEvents']>;
    getEventLoader: PoolMethod<ConfiguredProps, Adapter['getEventLoader']>;
}
export interface AdapterFunctions<ConfiguredProps extends {}> {
    beginIncrementalImport?: PoolMethod<ConfiguredProps, Adapter['beginIncrementalImport']>;
    commitIncrementalImport?: PoolMethod<ConfiguredProps, Adapter['commitIncrementalImport']>;
    dispose: PoolMethod<ConfiguredProps, Adapter['dispose']>;
    dropSnapshot?: PoolMethod<ConfiguredProps, Adapter['dropSnapshot']>;
    freeze: PoolMethod<ConfiguredProps, Adapter['freeze']>;
    getLatestEvent?: PoolMethod<ConfiguredProps, Adapter['getLatestEvent']>;
    injectEvent?: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['injectEvent']>;
    injectEvents: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['injectEvents']>;
    loadEventsByCursor: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['loadEventsByCursor']>;
    loadEventsByTimestamp: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['loadEventsByTimestamp']>;
    loadSnapshot?: PoolMethod<ConfiguredProps, Adapter['loadSnapshot']>;
    pushIncrementalImport?: PoolMethod<ConfiguredProps, Adapter['pushIncrementalImport']>;
    rollbackIncrementalImport?: PoolMethod<ConfiguredProps, Adapter['rollbackIncrementalImport']>;
    saveEvent: PoolMethod<ConfiguredProps, Adapter['saveEvent']>;
    saveSnapshot?: PoolMethod<ConfiguredProps, Adapter['saveSnapshot']>;
    shapeEvent: ShapeEvent;
    unfreeze: PoolMethod<ConfiguredProps, Adapter['unfreeze']>;
    getSecret: PoolMethod<ConfiguredProps, GetSecret>;
    setSecret: PoolMethod<ConfiguredProps, SetSecret>;
    deleteSecret: PoolMethod<ConfiguredProps, DeleteSecret>;
    loadSecrets?: PoolMethod<ConfiguredProps, Adapter['loadSecrets']>;
    injectSecret?: PoolMethod<ConfiguredProps, NonNullable<AdapterPoolBoundProps['injectSecret']>>;
    initEvents: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['initEvents']>;
    initSecrets: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['initSecrets']>;
    initFinal: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['initFinal']>;
    dropEvents: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['dropEvents']>;
    dropSecrets: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['dropSecrets']>;
    dropFinal: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['dropFinal']>;
    ensureEventSubscriber: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['ensureEventSubscriber']>;
    removeEventSubscriber: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['removeEventSubscriber']>;
    getEventSubscribers: PoolMethod<ConfiguredProps, AdapterPoolBoundProps['getEventSubscribers']>;
    replicateEvents?: PoolMethod<ConfiguredProps, Adapter['replicateEvents']>;
    replicateSecrets?: PoolMethod<ConfiguredProps, Adapter['replicateSecrets']>;
    setReplicationStatus?: PoolMethod<ConfiguredProps, Adapter['setReplicationStatus']>;
    setReplicationPaused?: PoolMethod<ConfiguredProps, Adapter['setReplicationPaused']>;
    getReplicationState?: PoolMethod<ConfiguredProps, Adapter['getReplicationState']>;
    resetReplication?: PoolMethod<ConfiguredProps, Adapter['resetReplication']>;
    setReplicationLock?: PoolMethod<ConfiguredProps, Adapter['setReplicationLock']>;
    getCursorUntilEventTypes?: PoolMethod<ConfiguredProps, Adapter['getCursorUntilEventTypes']>;
    describe: PoolMethod<ConfiguredProps, Adapter['describe']>;
    establishTimeLimit?: PoolMethod<ConfiguredProps, Adapter['establishTimeLimit']>;
    getEventLoaderNative?: PoolMethod<ConfiguredProps, NonNullable<AdapterPoolBoundProps['getEventLoaderNative']>>;
    runtimeInfo: PoolMethod<ConfiguredProps, Adapter['runtimeInfo']>;
    setReconnectionMode?: PoolMethod<ConfiguredProps, Adapter['setReconnectionMode']>;
}
export interface EventLoader {
    readonly loadEvents: (limit: number) => Promise<StoredEventBatchPointer>;
    readonly close: () => Promise<void>;
    readonly cursor: InputCursor;
    readonly isNative: boolean;
}
export declare type EventLoaderOptions = {
    preferRegular: boolean;
};
export declare type AdapterRuntimeInfo = {
    connectionCount: number;
    disposed: boolean;
    [key: string]: any;
};
export declare type ReconnectionMode = {
    maxReconnectionTimes?: number;
    delayBeforeReconnection?: number;
};
export interface Adapter extends CoreEventstore {
    importEvents: (options?: Partial<ImportOptions>) => ImportEventsStream;
    exportEvents: (options?: Partial<ExportOptions>) => ExportEventsStream;
    getLatestEvent: (filter: LatestEventFilter) => Promise<StoredEvent | null>;
    init: () => Promise<void>;
    drop: () => Promise<void>;
    dispose: () => Promise<void>;
    freeze: () => Promise<void>;
    unfreeze: () => Promise<void>;
    getSecretsManager: () => SecretsManager;
    dropSnapshot: (snapshotKey: string) => Promise<void>;
    pushIncrementalImport: (events: VersionlessEvent[], importId: string) => Promise<void>;
    beginIncrementalImport: () => Promise<string>;
    commitIncrementalImport: (importId: string, validateAfterCommit?: any) => Promise<void>;
    rollbackIncrementalImport: () => Promise<void>;
    incrementalImport: (events: VersionlessEvent[]) => Promise<void>;
    loadSecrets: (filter: SecretFilter) => Promise<SecretsWithIdx>;
    importSecrets: (options?: Partial<ImportSecretsOptions>) => stream.Writable;
    exportSecrets: (options?: Partial<ExportSecretsOptions>) => stream.Readable;
    gatherSecretsFromEvents: (events: StoredEvent[]) => Promise<GatheredSecrets>;
    getCursorUntilEventTypes: (cursor: InputCursor, untilEventTypes: Array<InputEvent['type']>) => Promise<string>;
    establishTimeLimit: (getVacantTimeInMillis: () => number) => void;
    getEventLoader: (filter: EventLoaderFilter, options?: EventLoaderOptions) => Promise<EventLoader>;
    runtimeInfo: () => AdapterRuntimeInfo;
    setReconnectionMode: (mode: ReconnectionMode) => void;
    setMonitoring: (monitoring: Monitoring | null) => void;
}
